import sys

import logging

import traceback


import logging


_loglevel = logging.WARNING


_logger_name = __name__

logger = None
console_handler = None

errHandler = None

# Create a log format using Log Record attributes
fmt = logging.Formatter(
    "%(name)s: %(asctime)s | %(levelname)s | %(filename)s:%(lineno)s | %(process)d >>> %(message)s"
)

fmt2 = "%(asctime)s %(levelname)s %(message)s"

datefmt = "%Y.%m.%d %H:%M:%S"


def _proto():
    global _loglevel, logger, console_handler,fmt,fmt2,datefmt
    try:
        log_file = __main__.__file__
    except:
        log_file = "jgt"
    try:
        import __main__

        logging.basicConfig(
            filename="{0}.log".format(log_file),
            level=logging.INFO,
            format=fmt,
            datefmt=datefmt,
        )
        console_handler = logging.StreamHandler(sys.stdout)
        console_handler.setLevel(_loglevel)
        logger = logging.getLogger(_logger_name)
        logger.addHandler(console_handler)
        # return logger
    except Exception as e:
        print("Exception: {0}\n{1}".format(e, traceback.format_exc()))
        print("logging failed - dont worry")
        logger = None


def _add_error_handler():
    global logger, errHandler,fmt
    try:
        errHandler = logging.FileHandler("error.log")
        errHandler.setLevel(logging.ERROR)
        errHandler.setFormatter(fmt)
        logger.addHandler(errHandler)
    except:
        print("Failed to add error handler")
        pass




def set_log_level(loglevel: str = "WARNING", logger_name=""):
    global _loglevel, _logger_name, logger
    if logger_name == "":
        logger_name = _logger_name
    _loglevel = getattr(logging, loglevel)
    logger.setLevel(_loglevel)
    # console_handler.setLevel(_loglevel)
    logger.info(f"Log level set to {_loglevel}")


def write_log(msg: str, loglevel: str = "INFO"):
    global _loglevel, logger
    loglevel = getattr(logging, loglevel)
    if loglevel >= _loglevel:
        logger.log(loglevel, msg)


def info(msg: str,*args):
  logger.info(msg,*args)
  #write_log(msg, "INFO")


def warning(msg: str,*args):
  logger.warning(msg,*args)
  


def error(msg: str,*args):
  logger.error(msg,*args)


def critical(msg: str,*args):
  logger.critical(msg,*args)


def debug(msg: str,*args):
  logger.debug(msg,*args)




try: 

    #_proto()
    #_add_error_handler() #BUGGED

    if logger is None:
        logger = logging.getLogger(_logger_name)
    # Create a logger object
    # log = logging.getLogger("jgt.log")
    #logger.setLevel(_loglevel)
except:
    print("Failed to create logger object. Dont worry")


import pytest
from jgtutils import module

def test_function():
    result = module.function()  # replace 'function' with your actual function
    expected_result =  # add your expected result here
    assert result == expected_result


# Copyright 2019 Gehtsoft USA LLC

# Licensed under the license derived from the Apache License, Version 2.0 (the "License"); 
# you may not use this file except in compliance with the License.

# You may obtain a copy of the License at

# http://fxcodebase.com/licenses/open-source/license.html

# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import __main__
import datetime
import traceback
import argparse
import sys

from forexconnect import fxcorepy

logging.basicConfig(filename='{0}.log'.format(__main__.__file__), level=logging.INFO,
                    format='%(asctime)s %(levelname)s %(message)s', datefmt='%m.%d.%Y %H:%M:%S')
console = logging.StreamHandler(sys.stdout)
console.setLevel(logging.INFO)
logging.getLogger('').addHandler(console)


def add_main_arguments(parser: argparse.ArgumentParser):
    parser.add_argument('-l',
                        metavar="LOGIN",
                        required=True,
                        help='Your user name.')

    parser.add_argument('-p',
                        metavar="PASSWORD",
                        required=True,
                        help='Your password.')

    parser.add_argument('-u',
                        metavar="URL",
                        required=True,
                        help='The server URL. For example,\
                                 https://www.fxcorporate.com/Hosts.jsp.')

    parser.add_argument('--connection',
                        metavar="CONNECTION",
                        required=False,
                        default="Demo",
                        help='The connection name. For example, \
                                 "Demo" or "Real".')

    parser.add_argument('-session',
                        help='The database name. Required only for users who\
                                 have accounts in more than one database.\
                                 Optional parameter.')

    parser.add_argument('-pin',
                        help='Your pin code. Required only for users who have \
                                 a pin. Optional parameter.')

def add_candle_open_price_mode_argument(parser: argparse.ArgumentParser):
    parser.add_argument('-o',
                        metavar="CANDLE_OPEN_PRICE_MODE",
                        default="prev_close",
                        help='Ability to set the open price candles mode. \
                        Possible values are first_tick, prev_close. For more information see description \
                        of O2GCandleOpenPriceMode enumeration. Optional parameter.')

def add_instrument_timeframe_arguments(parser: argparse.ArgumentParser, timeframe: bool = True):
    parser.add_argument('-i',
                        metavar="INSTRUMENT",
                        default="EUR/USD",
                        help='An instrument which you want to use in sample. \
                                  For example, "EUR/USD".')

    if timeframe:
        parser.add_argument('-t',
                            metavar="TIMEFRAME",
                            default="H1",
                            help='Time period which forms a single candle. \
                                      For example, m1 - for 1 minute, H1 - for 1 hour.')


def add_direction_rate_lots_arguments(parser: argparse.ArgumentParser, direction: bool = True, rate: bool = True,
                                      lots: bool = True):
    if direction:
        parser.add_argument('-d', metavar="TYPE", required=True,
                            help='The order direction. Possible values are: B - buy, S - sell.')
    if rate:
        parser.add_argument('-r', metavar="RATE", required=True, type=float,
                            help='Desired price of an entry order.')
    if lots:
        parser.add_argument('-lots', metavar="LOTS", default=1, type=int,
                            help='Trade amount in lots.')


def add_account_arguments(parser: argparse.ArgumentParser):
    parser.add_argument('-account', metavar="ACCOUNT",
                        help='An account which you want to use in sample.')


def valid_datetime(check_future: bool):
    def _valid_datetime(str_datetime: str):
        date_format = '%m.%d.%Y %H:%M:%S'
        try:
            result = datetime.datetime.strptime(str_datetime, date_format).replace(
                tzinfo=datetime.timezone.utc)
            if check_future and result > datetime.datetime.utcnow().replace(tzinfo=datetime.timezone.utc):
                msg = "'{0}' is in the future".format(str_datetime)
                raise argparse.ArgumentTypeError(msg)
            return result
        except ValueError:
            now = datetime.datetime.now()
            msg = "The date '{0}' is invalid. The valid data format is '{1}'. Example: '{2}'".format(
                str_datetime, date_format, now.strftime(date_format))
            raise argparse.ArgumentTypeError(msg)
    return _valid_datetime


def add_date_arguments(parser: argparse.ArgumentParser, date_from: bool = True, date_to: bool = True):
    if date_from:
        parser.add_argument('-datefrom',
                            metavar="\"m.d.Y H:M:S\"",
                            help='Date/time from which you want to receive\
                                      historical prices. If you leave this argument as it \
                                      is, it will mean from last trading day. Format is \
                                      "m.d.Y H:M:S". Optional parameter.',
                            type=valid_datetime(True)
                            )
    if date_to:
        parser.add_argument('-dateto',
                            metavar="\"m.d.Y H:M:S\"",
                            help='Datetime until which you want to receive \
                                      historical prices. If you leave this argument as it is, \
                                      it will mean to now. Format is "m.d.Y H:M:S". \
                                      Optional parameter.',
                            type=valid_datetime(False)
                            )


def add_report_date_arguments(parser: argparse.ArgumentParser, date_from: bool = True, date_to: bool = True):
    if date_from:
        parser.add_argument('-datefrom',
                            metavar="\"m.d.Y H:M:S\"",
                            help='Datetime from which you want to receive\
                                      combo account statement report. If you leave this argument as it \
                                      is, it will mean from last month. Format is \
                                      "m.d.Y H:M:S". Optional parameter.',
                            type=valid_datetime(True)
                            )
    if date_to:
        parser.add_argument('-dateto',
                            metavar="\"m.d.Y H:M:S\"",
                            help='Datetime until which you want to receive \
                                      combo account statement report. If you leave this argument as it is, \
                                      it will mean to now. Format is "m.d.Y H:M:S". \
                                      Optional parameter.',
                            type=valid_datetime(True)
                            )


def add_max_bars_arguments(parser: argparse.ArgumentParser):
    parser.add_argument('-quotescount',
                        metavar="MAX",
                        default=0,
                        type=int,
                        help='Max number of bars. 0 - Not limited')


def add_bars_arguments(parser: argparse.ArgumentParser):
    parser.add_argument('-bars',
                        metavar="COUNT",
                        default=3,
                        type=int,
                        help='Build COUNT bars. Optional parameter.')


def print_exception(exception: Exception):
    logging.error("Exception: {0}\n{1}".format(exception, traceback.format_exc()))


# function for print available descriptors
def session_status_changed(session: fxcorepy.O2GSession,
                           status: fxcorepy.AO2GSessionStatus.O2GSessionStatus):
    logging.info("Status: " + str(status))
    if status == fxcorepy.AO2GSessionStatus.O2GSessionStatus.TRADING_SESSION_REQUESTED:
        descriptors = session.trading_session_descriptors
        logging.info("Session descriptors:")
        logging.info(" {0:>7} | {1:>7} | {2:>30} | {3:>7}\n".format("id", "name", "description", "requires pin"))
        for desc in descriptors:
            logging.info(" {0:>7} | {1:>7} | {2:>30} | {3:>7}\n".format(desc.id, desc.name,
                                                                        desc.description,
                                                                        str(desc.requires_pin)))


def diff_month(year: int, month: int, date2: datetime):
    return (year - date2.year) * 12 + month - date2.month


def convert_timeframe_to_seconds(unit: fxcorepy.O2GTimeFrameUnit, size: int):
    current_unit = unit
    current_size = size
    step = 1
    if current_unit == fxcorepy.O2GTimeFrameUnit.MIN:
        step = 60  # leads to seconds
    elif current_unit == fxcorepy.O2GTimeFrameUnit.HOUR:
        step = 60*60
    elif current_unit == fxcorepy.O2GTimeFrameUnit.DAY:
        step = 60*60*24
    elif current_unit == fxcorepy.O2GTimeFrameUnit.WEEK:
        step = 60*60*24*7
    elif current_unit == fxcorepy.O2GTimeFrameUnit.MONTH:
        step = 60 * 60 * 24 * 30
    elif current_unit == fxcorepy.O2GTimeFrameUnit.TICK:
        step = 1
    return step * current_size



# %% Imports


import argparse
from jgtpov import calculate_tlid_range as get_tlid_range

# %%
result = get_tlid_range('2021-01-01', 'H4', 1000)
result
# %%
result = get_tlid_range('21-01-01', 'H4', 1000)
result
# %%
result
# %%

# %%
result = get_tlid_range('2021-01-01', 'D1', 1000)

# %%
result
# %%
# %%
result = get_tlid_range('2021-01-01', 'W1', 1000)

# %%
result
# %%
# %%
result = get_tlid_range('2021-01-01', 'M1', 500)

# %%
result
# %%


import subprocess

import os
import sys
sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))

import jgtos

import platform

def pwsd_wsl_run_command1(bash_command_to_run):
    powershell_command = 'wsl.exe bash -c \'' + bash_command_to_run + '\''
    result = subprocess.run(
        ["pwsh.exe", "-Command", powershell_command], stdout=subprocess.PIPE, shell=True
    )
    return result.stdout.decode("utf-8")





def run_bash_command_by_platform(bash_cmd):
    try:
        if platform.system() == "Windows":
            shell = os.environ.get('COMSPEC', 'cmd.exe')
            if 'powershell' in shell.lower():
                # The interpreter is PowerShell            
                return subprocess.run(bash_cmd, shell=True, stdout=subprocess.PIPE).stdout.decode("utf-8")
            else:
                # The interpreter is cmd.exe
                return wsl_run_bash_on_cmd(bash_cmd)
        else:
            # The system is Linux
            return subprocess.run(bash_cmd, shell=True, stdout=subprocess.PIPE).stdout.decode("utf-8")
    except Exception as e:
        print(f"An error occurred running jgtfxcli: {str(e)}")
        print(f"   bash_cmd: {bash_cmd}")
        raise e
        #return None
    
def wsl_run_bash_on_cmd(bash_cmd):   
    
    powershell_command = 'wsl.exe bash -c \'' + bash_cmd + '\''
    result = subprocess.run(
        ["pwsh.exe", "-Command", powershell_command], stdout=subprocess.PIPE, shell=True
    )
    return result.stdout.decode("utf-8")



def run(bash_command):
    return run_bash_command_by_platform(bash_command)


    

def resolve_cli_path(cli_path):
    if cli_path == "" or cli_path is None or cli_path == 0 or cli_path == '0':
        cli_path = '$HOME/.local/bin/jgtfxcli'
    if not os.path.exists(cli_path):
        cli_path = 'jgtfxcli'    
    
    return cli_path #@STCIssue Should install : pip install --user jgtfxcon    (if not found)

def jgtfxcli_wsl(instrument:str, timeframe:str, quote_count:int,cli_path="", verbose_level=0,use_full=False):
    cli_path=resolve_cli_path(cli_path)
    if cli_path == "" or cli_path is None or cli_path == 0 or cli_path == '0':
        cli_path = '$HOME/.local/bin/jgtfxcli'
        #cli_path = "/home/jgi/.local/bin/jgtfxcli"
    if use_full:
        bash_command_to_run = f"pwd;{cli_path} -i \"{instrument}\" -t \"{timeframe}\" --full  -v {verbose_level} "
    else :
        bash_command_to_run = f"pwd;{cli_path} -i \"{instrument}\" -t \"{timeframe}\" -c {quote_count} -v {verbose_level}"
    if verbose_level > 0:
        print(f"bash_command_to_run: {bash_command_to_run}")
    
    return run_bash_command_by_platform(bash_command_to_run)


def _mkbash_cmd_string_jgtfxcli_range(instrument:str, timeframe:str,tlid_range=None,cli_path="", verbose_level=0,quote_count=420,use_full=False,keep_bid_ask=False):
    cli_path=resolve_cli_path(cli_path)
    
    
    #env variable bypass if env exist JGT_KEEP_BID_ASK=1, keep_bid_ask = True
    bidask_arg = " "
    if os.getenv("JGT_KEEP_BID_ASK","0") == "1":
        keep_bid_ask = True
    if keep_bid_ask:
        bidask_arg = " -kba "
    if tlid_range is not None:
        bash_command_to_run = f"pwd;{cli_path} -i \"{instrument}\" -t \"{timeframe}\" -r \"{tlid_range}\" -v {verbose_level} {bidask_arg}"
    else:
        if use_full:
            bash_command_to_run = f"pwd;{cli_path} -i \"{instrument}\" -t \"{timeframe}\" --full  -v {verbose_level}  {bidask_arg}"
        else:
            bash_command_to_run = f"pwd;{cli_path} -i \"{instrument}\" -t \"{timeframe}\" -c \"{quote_count}\" -v {verbose_level}  {bidask_arg}"
        
    
    return bash_command_to_run

def _mkbash_cmd_string_jgtfxcli_range1(instrument:str, timeframe:str,tlid_range=None,cli_path="", verbose_level=0):
    cli_path=resolve_cli_path(cli_path)
    
    date_from,date_to=jgtos.tlid_range_to_jgtfxcon_start_end_str(tlid_range)
    
    bash_command_to_run = f"pwd;{cli_path} -i \"{instrument}\" -t \"{timeframe}\" -s \"{date_from}\" -e \"{date_to}\" -v {verbose_level}"
    return bash_command_to_run

def jgtfxcli_wsl_range(instrument:str, timeframe:str, quote_count:int,tlid_range=None,cli_path="", verbose_level=0,use_full=False,keep_bid_ask=False):
    bash_command_to_run = _mkbash_cmd_string_jgtfxcli_range(instrument, timeframe,tlid_range,cli_path, verbose_level,quote_count,use_full=use_full)
        
    return run_bash_command_by_platform(bash_command_to_run)

def jgtfxcli(instrument:str, timeframe:str, quote_count:int,cli_path="", verbose_level=0,use_full=False):
    return jgtfxcli_wsl(instrument,timeframe,quote_count,cli_path,verbose_level,use_full=use_full)

def getPH(instrument:str, timeframe:str, quote_count:int,tlid_range=None, verbose_level=0,use_full=False,keep_bid_ask=False):
    return jgtfxcli_wsl_range(instrument, timeframe, quote_count,tlid_range,"", verbose_level,use_full=use_full,keep_bid_ask=keep_bid_ask)


def wsl_cd(directory):
    # Define the command to be executed
    command = ["wsl.exe", "cd", directory]

    # Execute the command
    result = subprocess.run(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)


    # Print the error (if any)
    if result.stderr:
        return result.stderr.decode("utf-8")
    else:
        # Print the output
        return result.stdout.decode("utf-8")
        

def cd(tpath):
    wsl_cd(tpath)

def execute_wsl_command_v1_with_cd(directory, command_to_execute):
    # Define the command to be executed
    command = ["wsl.exe", "cd", directory, "&&", command_to_execute]

    # Execute the command
    result = subprocess.run(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)

    # Print the output
    print(result.stdout.decode("utf-8"))

    # Print the error (if any)
    if result.stderr:
        print("Error:", result.stderr.decode("utf-8"))


minimum_quote_count = 335

# Define the dividers for each timeframe
timeframe_dividers = {
    "m1": 0.0166,
    "mi1": 0.0166,
    "m5": 0.8,
    "m15": 0.25,
    "m30": 0.5,
    "H1": 1,
    "H2": 2,
    "H3": 3,
    "H4": 4,
    "H5": 5,
    "H6": 6,
    "H8": 8,
    "D1": 24,
    "W1": 110,
    "M1": 400
}

#@STCIssue Locks us to H1, should be interactive and receive an input from the user, lower TF
def get_timeframe_dividers(base_tf="H1"):
    return timeframe_dividers[base_tf]



import datetime

import sys
import os
sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))



def create_filestore_path(
    instrument:str, 
    timeframe:str, 
    quiet=True, 
    compressed=False, 
    tlid_range:str=None,
    output_path:str=None,
    nsdir:str="pds",
    use_full=False
):
    # Define the file path based on the environment variable or local path
    if output_path is None:
        data_path = get_data_path(nsdir=nsdir,use_full=use_full)
    else: # get path from var in os
        data_path = os.path.join (output_path.replace("/"+nsdir,""),nsdir)
    
    if not quiet:
        print(data_path)

    ext = "csv"
    if compressed:
        ext = "csv.gz"

    fpath = mk_fullpath(instrument, timeframe, ext, data_path, tlid_range=tlid_range)

    if os.name == "nt":
        fpath = fpath.replace("/", "\\")
    return fpath


def mk_fn(instrument:str, 
          timeframe:str, 
          ext:str="csv"):
    """Make a file name with instrument and timeframe

    Args:
        instrument (str): symbol
        timeframe (str): TF
        ext (str): ext name "csv"

    Returns:
        str: file name
    """
    _tf = timeframe
    _i = instrument.replace("/", "-")
    if timeframe == "m1":
        _tf = timeframe.replace("m1", "mi1")
    _fn = _i + "_" + _tf + "." + ext
    return _fn.replace("..", ".")


def mk_fn_range(instrument:str, 
                timeframe:str, 
                start: datetime.datetime, 
                end: datetime.datetime, 
                ext="csv"):
    _tf = timeframe
    _i = instrument.replace("/", "-")
    if timeframe == "m1":
        _tf = timeframe.replace("m1", "mi1") #differenciate with M1
    start_str = tlid_dt_to_string(start)
    end_str = tlid_dt_to_string(end)
    _fn = f"{_i}_{_tf}_{start_str}_{end_str}.{ext}"
    # _fn= _i + '_' + _tf + '.' + ext
    _fn = _fn.replace("..", ".")
    _fn = _fn.replace("/", "-")
    return _fn

import subprocess

def create_directory_with_sudo(path):
    # Construct the command
    command = f"sudo mkdir -p -m 777 {path}"

    # Execute the command
    subprocess.run(command, shell=True)



def mk_fullpath(instrument:str, 
                timeframe:str, 
                ext:str, 
                path:str, 
                tlid_range:str=None):
    #path dont exist, try to make that directory
    if not os.path.exists(path):
        try:
            os.makedirs(path, exist_ok=True)
        except:
            print("Error creating the directory : " + path)
            print("Executing:  sudo mkdir -m 777 -p " + path)
            #execute system command sudo mkdir -p -m 777 $path
            try:
                create_directory_with_sudo(path)
                print("Directory created : " + path)
            except:
                print("Error creating the directory : " + path)
                print("Please create the directory manually:  sudo mkdir -m 777 -p " + path)
            
            
    if tlid_range is None:
        fn = mk_fn(instrument, timeframe, ext)
    else:
        start_dt, end_dt = tlid_range_to_start_end_datetime(
            tlid_range=tlid_range
        )
        # print(str(start_dt),str(end_dt))
        fn = mk_fn_range(instrument, timeframe, start_dt, end_dt, ext)
    rpath = os.path.join(path, fn)
    if os.name == "nt":
        rpath = rpath.replace("/", "\\")
    # path + '/'+fn
    return rpath




def get_data_path(nsdir: str, 
                  range_level:int=6,
                  use_full=False):
    # Try to read the path from the JGTPY_DATA environment variable
    if use_full:
        default_data_full="/var/lib/jgt/full/data"
        data_path = os.environ.get('JGTPY_DATA_FULL',default_data_full)

        #create the directory if it does not exist
        try:
            os.makedirs(data_path, exist_ok=True)
        except:
            print("Error creating the directory : " + data_path)
            print("Please create the directory manually:  sudo mkdir -m 777 -p " + data_path)
            data_path = "/data/full"
            print("Using default directory : " + data_path)
            os.makedirs(data_path, exist_ok=True)


        if not os.path.exists(data_path):
            print("Once the directory is created, you can generate the PDS/CDS manually : " + data_path)
            
            
                
    else:
        default_data="/var/lib/jgt/data"
        data_path = os.environ.get('JGTPY_DATA',default_data)
        #create the directory if it does not exist 
        try:        
            os.makedirs(data_path, exist_ok=True)
        except:                      
            print("Error creating the directory : " + data_path)                                                                          
            print("Please create the directory manually:  sudo mkdir -m 777 -p " + data_path) 
            data_path = "/data" 
            print("Using default directory : " + data_path) 
            os.makedirs(data_path, exist_ok=True) 
        
    # If the variable is defined and the path exists, return it
    if data_path and os.path.exists(data_path):
        rpath = os.path.abspath(os.path.join(data_path, nsdir))
        return rpath

    # If the variable is not defined or the path does not exist, fall back to the current behavior
    # Start with the current working directory
    base_path = os.getcwd()

    # Try up to X range levels up to find the data directory
    for _ in range(range_level):
        data_path = os.path.join(base_path, 'data')
        if os.path.exists(data_path):
            # Check if the directory has write permissions
            if not os.access(data_path, os.W_OK):
                raise Exception(f"No write access to the directory: {data_path}")
            break
        # Go one level up for the next iteration
        base_path = os.path.abspath(os.path.join(base_path, '..'))
    else:
        # If the loop completes without finding the data directory, raise an exception
        raise Exception("Data directory not found. Please create a directory named 'data' in the current, parent directory (up to 3 levels), or set the JGTPY_DATA environment variable.")

    # Replace slashes with backslashes on Windows
    if os.name == "nt":
        data_path = data_path.replace("/", "\\")

    # Append the nsdir to the data path
    data_path = os.path.abspath(os.path.join(data_path, nsdir))

    return data_path


def tlid_range_to_start_end_datetime(tlid_range: str):
    #Support inputting just a Year
    if len(tlid_range) == 4 or len(tlid_range) == 2 :
        start_str = tlid_range + "0101" + "0000"
        end_str = tlid_range +  "1231" + "2359"
    else:
        #Normal support start_end
        try:
            start_str, end_str = tlid_range.split("_")
        except:
            print("TLID ERROR - make use you used a \"_\"")
            return None,None
    
    date_format_start = "%y%m%d%H%M"
    date_format_end = "%y%m%d%H%M"
    
    if len(start_str) == 4 or len(start_str) == 2:
        start_str = start_str + "0101" + "0000"
    if len(end_str) == 4 or len(end_str) == 2 :
        end_str = end_str + "1231" + "2359"
    
    if len(start_str) == 6:
        start_str = start_str + "0000"
    if len(end_str) == 6:
        end_str = end_str + "2359"
   
    if len(start_str) == 8:
        start_str = start_str + "0000"
    if len(end_str) == 8:
        end_str = end_str + "2359"
        
    if len(start_str) == 12:
        date_format_start = "%Y%m%d%H%M"
    if len(end_str) == 12:
        date_format_end = "%Y%m%d%H%M"
   
    #print(date_format_end)
    try:
        start_dt =  datetime.datetime.strptime(start_str, date_format_start)
        end_dt = datetime.datetime.strptime(end_str, date_format_end)
        return start_dt,end_dt
    except ValueError:
        return None

def tlid_range_to_jgtfxcon_start_end_str(tlid_range: str):
    date_format_fxcon = '%m.%d.%Y %H:%M:%S'
    start_dt,end_dt = tlid_range_to_start_end_datetime(tlid_range)
    #print(str(start_dt),str(end_dt))
    if start_dt is None or end_dt is None:
        return None,None
    else:
        return str(start_dt.strftime(date_format_fxcon)),str(end_dt.strftime(date_format_fxcon))

def tlid_dt_to_string(dt: datetime.datetime):
    return dt.strftime("%y%m%d%H%M")

def tlidmin_to_dt(tlid_str: str):
    date_format = "%y%m%d%H%M"
    try:
        tlid_dt =  datetime.datetime.strptime(tlid_str, date_format)
        return tlid_dt
    except ValueError:
        pass
    
    return None
